/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.agent.jcmd.backend.internal;

import com.redhat.thermostat.agent.utils.management.MXBeanConnection;
import com.redhat.thermostat.agent.utils.management.MXBeanConnectionPool;
import com.redhat.thermostat.common.utils.LoggingUtils;
import com.redhat.thermostat.agent.jcmd.backend.internal.model.JCMDBinding;
import com.redhat.thermostat.lang.schema.models.Pid;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

public class JMXConnectionManager {
    private static final Logger log = LoggingUtils.getLogger(JMXConnectionManager.class);

    private Map<Pid, JCMDBinding> connections;
    private MXBeanConnectionPool mxBeanPool;

    JMXConnectionManager(Map<Pid, JCMDBinding> connections, MXBeanConnectionPool mxBeanPool) {
        this.connections = connections;
        this.mxBeanPool = mxBeanPool;
    }

    JMXConnectionManager(MXBeanConnectionPool mxBeanPool) {
        this(new HashMap<Pid, JCMDBinding>(), mxBeanPool);
    }

    public JCMDBinding getBindings(Pid pid) throws Exception {

        if (connections.containsKey(pid)) {
            return connections.get(pid);
        }

        try {
            MXBeanConnection connection = mxBeanPool.acquire(pid.get());

            JCMDBinding binding = new JCMDBinding(connection);

            if (binding.canCollectDiagnosticData()) {
                connections.put(pid, binding);

            } else {
                log.warning("vm does not run with -XX:+UnlockDiagnosticVMOptions");
                mxBeanPool.release(pid.get(), connection);
                throw new UnsupportedOperationException("vm does not run with +UnlockDiagnosticVMOptions");
            }

            return binding;

        } catch (NumberFormatException nfe) {
            log.warning("vm-pid cannot be parsed: " + pid);
            throw nfe;
        }
    }

    public void release(Pid pid) {

    }
}
