/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.vm.cpu.agent.model;

import java.io.IOException;
import java.util.List;

import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;

public class VmCpuStatTypeAdapter extends TypeAdapter<List<VmCpuStat>> {

    private static final String TYPE_LONG = "$numberLong";
    private static final String TIMESTAMP = "timeStamp";
    private static final String VM_ID = "jvmId";
    private static final String CPU_LOAD = "cpuLoad";
    private static final String AGENT_ID = "agentId";

    private void writeMemoryStat(JsonWriter out, VmCpuStat stat) throws IOException {
        out.beginObject();
        out.name(TIMESTAMP);
        writeLong(out, stat.getTimeStamp());
        out.name(VM_ID);
        out.value(stat.getVmId());
        out.name(CPU_LOAD);
        out.value(stat.getCpuLoad());
        out.name(AGENT_ID);
        out.value(stat.getAgentId());
        out.endObject();
    }

    private void writeLong(JsonWriter out, long input) throws IOException {
        out.beginObject();
        out.name(TYPE_LONG);
        out.value(String.valueOf(input));
        out.endObject();
    }

    @Override
    public void write(JsonWriter jsonWriter, List<VmCpuStat> vmCpuStats) throws IOException {
        jsonWriter.beginArray();
        for (VmCpuStat stat : vmCpuStats) {
            writeMemoryStat(jsonWriter, stat);
        }
        jsonWriter.endArray();
    }

    @Override
    public List<VmCpuStat> read(JsonReader jsonReader) throws IOException {
        return null;
    }
}
